@@ -55,14 +55,26 @@ using namespace std;
 # define WIN32_CAST_CHAR (LPCWSTR)
 #endif
 
+#ifndef _WIN32_WCE
 static HINSTANCE kdecoreDllInstance = NULL;
-static wchar_t kde4prefixUtf16[MAX_PATH + 2];
+#else
+static HANDLE kdecoreDllInstance = NULL;
+#endif
+#ifdef KDELIBS_STATIC_LIBS
+static bool kde4prefixInitialized = false;
+#endif
+static wchar_t kde4prefixUtf16[MAX_PATH + 2] = L"";
+
 static QString *kde4Prefix = NULL;
 
 void initKde4prefixUtf16()
 {
     //the path is C:\some\path\kde4\bin\kdecore.dll
+#ifndef _WIN32_WCE
     GetModuleFileNameW(kdecoreDllInstance, kde4prefixUtf16, MAX_PATH + 1);
+#else
+    GetModuleFileNameW((HMODULE)kdecoreDllInstance, kde4prefixUtf16, MAX_PATH + 1);
+#endif
     int bs1 = 0, bs2 = 0;
 
     //we convert \ to / and remove \bin\kdecore.dll from the string
@@ -84,16 +96,46 @@ void initKde4prefixUtf16()
 // don't have an instantiated QCoreApplication
 QString getKde4Prefix()
 {
+#ifdef _WIN32_WCE
+    if (kde4prefixInitialized)
+        return QString::fromUtf16((ushort*) kde4prefixUtf16);
+    
+    QDir kde4prefixDir(QString::fromUtf16((ushort*) STATIC_INSTALL_PATH));
+    if (kde4prefixDir.exists()){
+        wcscpy(kde4prefixUtf16, STATIC_INSTALL_PATH);
+        kde4prefixUtf16[wcslen(kde4prefixUtf16)] = 0;       
+        kde4prefixInitialized = true;
+        return QString::fromUtf16((ushort*) kde4prefixUtf16);
+    } else {
+        bool ok;
+        QString retval = getWin32RegistryValue(HKEY_LOCAL_MACHINE, "Software\\kde", "KDEDIRS", &ok);
+        if (!ok){
+            return QString();
+        } else {
+            retval = QDir::fromNativeSeparators(retval);
+            wcscpy(kde4prefixUtf16, retval.utf16());
+            kde4prefixUtf16[wcslen(kde4prefixUtf16)] = 0;
+            kde4prefixInitialized = true;
+            return retval;
+        }
+    }
+#else
   // we can get called after DLL_PROCESS_DETACH!
   return kde4Prefix ? *kde4Prefix : QString::fromUtf16((ushort*) kde4prefixUtf16);
+#endif
 }
 
+#ifndef KDELIBS_STATIC_LIBS
 /**
  * The dll entry point - get the instance handle for GetModuleFleNameW
  * Maybe also some special initialization / cleanup can be done here
  **/
 extern "C"
+#ifndef _WIN32_WCE
 BOOL WINAPI DllMain ( HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpReserved)
+#else
+BOOL WINAPI DllMain ( HANDLE hinstDLL,DWORD fdwReason,LPVOID lpReserved)
+#endif
 {
     switch ( fdwReason ) {
     case DLL_PROCESS_ATTACH:
@@ -123,6 +165,8 @@ BOOL WINAPI DllMain ( HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpReserved)
     return true;
 }
 
+#endif
+
 /**
  \return a value from MS Windows native registry.
  @param key is usually one of HKEY_CLASSES_ROOT, HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE
@@ -133,7 +177,7 @@ BOOL WINAPI DllMain ( HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpReserved)
  @param item is an item inside subKey or "" if default folder's value should be returned
  @param ok if not null, will be set to true on success and false on failure
 */
-QString getWin32RegistryValue ( HKEY key, const QString& subKey, const QString& item, bool *ok = 0 )
+QString getWin32RegistryValue ( HKEY key, const QString& subKey, const QString& item, bool *ok )
 {
 #define FAILURE \
  { if (ok) \
@@ -175,14 +219,28 @@ bool showWin32FilePropertyDialog ( const QString& fileName )
 {
     QString path_ = QDir::convertSeparators ( QFileInfo ( fileName ).absoluteFilePath() );
 
+#ifndef _WIN32_WCE
     SHELLEXECUTEINFOW execInfo;
+#else
+    SHELLEXECUTEINFO execInfo;
+#endif
     memset ( &execInfo,0,sizeof ( execInfo ) );
     execInfo.cbSize = sizeof ( execInfo );
+#ifndef _WIN32_WCE
     execInfo.fMask = SEE_MASK_INVOKEIDLIST | SEE_MASK_NOCLOSEPROCESS | SEE_MASK_FLAG_NO_UI;
+#else
+    execInfo.fMask = SEE_MASK_NOCLOSEPROCESS | SEE_MASK_FLAG_NO_UI;
+#endif
     const QString verb ( QLatin1String ( "properties" ) );
     execInfo.lpVerb = WIN32_CAST_CHAR verb.utf16();
     execInfo.lpFile = WIN32_CAST_CHAR path_.utf16();
+#ifndef _WIN32_WCE
     return ShellExecuteExW ( &execInfo );
+#else
+    return ShellExecuteEx ( &execInfo );
+    //There is no native file property dialog in wince
+   // return false;
+#endif
 }
 
 // note: QLocale().name().left(2).toLatin1() returns the same
@@ -192,7 +250,7 @@ QByteArray getWin32LocaleName()
     bool ok;
     QString localeNumber = getWin32RegistryValue ( HKEY_CURRENT_USER,
                            QLatin1String("Control Panel\\International"),
-                           "Locale", &ok );
+                           QLatin1String("Locale"), &ok );
     if ( !ok )
         return QByteArray();
     QString localeName = getWin32RegistryValue ( HKEY_LOCAL_MACHINE,
@@ -240,7 +298,7 @@ static void kMessageOutputDebugString(QtMsgType type, const char *msg)
             break;
     }
     strlcat(buf,"\n",BUFSIZE);
-    OutputDebugStringA(buf);
+    OutputDebugStringW( (WCHAR*)QString::fromLatin1(buf).utf16());
     delete[] buf;
 }
 
@@ -287,6 +345,8 @@ static bool attachToConsole()
 */ 
 static void redirectToConsole()
 {
+//FIXME: for wince we cannot set stdio buffers
+#ifndef _WIN32_WCE
     int hCrt;
     FILE *hf;
     int i;
@@ -314,6 +374,7 @@ static void redirectToConsole()
     // make cout, wcout, cin, wcin, wcerr, cerr, wclog and clog
     // point to console as well
     ios::sync_with_stdio();
+#endif
 }
 
 #include <streambuf>
@@ -341,7 +402,7 @@ class debug_streambuf: public std::streambuf
                 if (cc == '\n')
                 {
                     buf[index] = '\0';
-                    OutputDebugStringA((LPCSTR)buf);
+                    OutputDebugStringW((WCHAR*)QString::fromLatin1(buf).utf16());
                     index = rindex;
                 }
             }
@@ -358,6 +419,10 @@ class debug_streambuf: public std::streambuf
 */
 static int subSystem()
 {
+#ifdef _WIN32_WCE
+    // there is only one subsystem on Windows CE
+    return IMAGE_SUBSYSTEM_WINDOWS_CE_GUI;
+#else
     static int subSystem = -1;
     if (subSystem > -1)
         return subSystem; 
@@ -372,6 +437,7 @@ static int subSystem()
     }
     subSystem = ntHeader->OptionalHeader.Subsystem;
     return subSystem;
+#endif
 }
     
 /**
@@ -391,6 +457,8 @@ static int subSystem()
     gui        yes       win32debug         console         console
     gui        no        win32debug         win32debug      win32debug 
 
+    win-ce     no        win32debug         win32debug      win32debug
+
 [1]no redirect solution for FILE * based output yet
 
  TODO: report events to the windows event log system 
@@ -432,6 +500,11 @@ static class kMessageOutputInstaller {
                     oldStderrBuffer = std::cerr.rdbuf(&stderrBuffer);
                     // TODO: redirect FILE * level to console, no idea how to do yet
                 }
+            } else if (subSystem() == IMAGE_SUBSYSTEM_WINDOWS_CE_GUI) {
+                // do not try to get a console on WinCE systems
+                qInstallMsgHandler(kMessageOutputDebugString);
+                oldStdoutBuffer = std::cout.rdbuf(&stdoutBuffer);
+                oldStderrBuffer = std::cerr.rdbuf(&stderrBuffer);
             }
             else
                 qWarning("unknown subsystem %d detected, could not setup qt message handler",subSystem());
