@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2002 Carsten Pfeiffer <pfeiffer@kde.org>
                  2005 Michael Brade <brade@kde.org>
+		 2012 Laurent Montel <montel@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -63,6 +64,8 @@ class KTextEdit::Private
         customPalette( false ),
         checkSpellingEnabled( false ),
         findReplaceEnabled(true),
+        showTabAction(true),
+	showAutoCorrectionButton(false),
         highlighter( 0 ), findDlg(0),find(0),repDlg(0),replace(0), findIndex(0), repIndex(0),
         lastReplacedPosition(-1)
     {
@@ -124,6 +127,7 @@ class KTextEdit::Private
 
     void init();
 
+    void checkSpelling(bool force);
     KTextEdit *parent;
     KTextEditSpellInterface *spellInterface;
     QAction *autoSpellCheckAction;
@@ -135,6 +139,8 @@ class KTextEdit::Private
 
     bool checkSpellingEnabled : 1;
     bool findReplaceEnabled: 1;
+    bool showTabAction: 1;
+    bool showAutoCorrectionButton: 1;
     QTextDocumentFragment originalDoc;
     QString spellCheckingConfigFileName;
     QString spellCheckingLanguage;
@@ -148,6 +154,55 @@ class KTextEdit::Private
     KConfig *sonnetKConfig;
 };
 
+void KTextEdit::Private::checkSpelling(bool force)
+{
+  if(parent->document()->isEmpty())
+  {
+      KMessageBox::information(parent, i18n("Nothing to spell check."));
+      if(force) {
+	emit parent->spellCheckingFinished();
+      }
+      return;
+  }
+  Sonnet::BackgroundChecker *backgroundSpellCheck = new Sonnet::BackgroundChecker;
+  if(!spellCheckingLanguage.isEmpty())
+     backgroundSpellCheck->changeLanguage(spellCheckingLanguage);
+  Sonnet::Dialog *spellDialog = new Sonnet::Dialog(
+      backgroundSpellCheck, force ? parent : 0);
+  backgroundSpellCheck->setParent(spellDialog);
+  spellDialog->setAttribute(Qt::WA_DeleteOnClose, true);
+  spellDialog->activeAutoCorrect(showAutoCorrectionButton);
+  connect(spellDialog, SIGNAL(replace(QString,int,QString)),
+          parent, SLOT(spellCheckerCorrected(QString,int,QString)));
+  connect(spellDialog, SIGNAL(misspelling(QString,int)),
+          parent, SLOT(spellCheckerMisspelling(QString,int)));
+  connect(spellDialog, SIGNAL(autoCorrect(QString,QString)),
+          parent, SLOT(spellCheckerAutoCorrect(QString,QString)));
+  connect(spellDialog, SIGNAL(done(QString)),
+          parent, SLOT(spellCheckerFinished()));
+  connect(spellDialog, SIGNAL(cancel()),
+          parent, SLOT(spellCheckerCanceled()));
+  //Laurent in sonnet/dialog.cpp we emit done(QString) too => it calls here twice spellCheckerFinished not necessary
+  /*
+  connect(spellDialog, SIGNAL(stop()),
+          parent, SLOT(spellCheckerFinished()));
+  */
+  connect(spellDialog, SIGNAL(spellCheckStatus(QString)),
+          parent, SIGNAL(spellCheckStatus(QString)));
+  connect(spellDialog, SIGNAL(languageChanged(QString)),
+          parent, SIGNAL(languageChanged(QString)));
+  if(force) {
+      connect(spellDialog, SIGNAL(done(QString)),parent, SIGNAL(spellCheckingFinished()));
+      connect(spellDialog, SIGNAL(cancel()), parent, SIGNAL(spellCheckingCanceled()));
+      //Laurent in sonnet/dialog.cpp we emit done(QString) too => it calls here twice spellCheckerFinished not necessary
+      //connect(spellDialog, SIGNAL(stop()), parent, SIGNAL(spellCheckingFinished()));
+  }
+  originalDoc = QTextDocumentFragment(parent->document());
+  spellDialog->setBuffer(parent->toPlainText());
+  spellDialog->show();
+}
+
+
 void KTextEdit::Private::spellCheckerCanceled()
 {
     QTextDocument *doc = parent->document();
@@ -157,9 +212,9 @@ void KTextEdit::Private::spellCheckerCanceled()
     spellCheckerFinished();
 }
 
-void KTextEdit::Private::spellCheckerAutoCorrect(const QString&,const QString&)
+void KTextEdit::Private::spellCheckerAutoCorrect(const QString& currentWord,const QString& autoCorrectWord)
 {
-    //TODO
+    emit parent->spellCheckerAutoCorrect(currentWord, autoCorrectWord);
 }
 
 void KTextEdit::Private::spellCheckerMisspelling( const QString &text, int pos )
@@ -495,9 +550,11 @@ QMenu *KTextEdit::mousePopupMenu()
       d->autoSpellCheckAction->setCheckable( true );
       d->autoSpellCheckAction->setChecked( checkSpellingEnabled() );
       popup->addSeparator();
-      d->allowTab = popup->addAction( i18n("Allow Tabulations") );
-      d->allowTab->setCheckable( true );
-      d->allowTab->setChecked( !tabChangesFocus() );
+      if (d->showTabAction) {
+        d->allowTab = popup->addAction( i18n("Allow Tabulations") );
+        d->allowTab->setCheckable( true );
+        d->allowTab->setChecked( !tabChangesFocus() );
+      }
   }
 
   if (d->findReplaceEnabled) {
@@ -613,12 +670,7 @@ void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
     // Use standard context menu for already selected words, correctly spelled
     // words and words inside quotes.
     if (!wordIsMisspelled || selectedWordClicked || inQuote) {
-        QMenu *popup = mousePopupMenu();
-        if ( popup ) {
-            aboutToShowContextMenu(popup);
-            popup->exec( event->globalPos() );
-            delete popup;
-        }
+        QMetaObject::invokeMethod(this, "mousePopupMenuImplementation", Q_ARG(QPoint, event->globalPos()));
     }
     else {
         QMenu menu; //don't use KMenu here we don't want auto management accelerator
@@ -630,7 +682,8 @@ void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
             suggestionsAction->setEnabled(false);
         }
         else {
-            for (QStringList::const_iterator it = reps.constBegin(); it != reps.constEnd(); ++it) {
+            QStringList::const_iterator end(reps.constEnd());
+            for (QStringList::const_iterator it = reps.constBegin(); it != end; ++it) {
                 menu.addAction(*it);
             }
         }
@@ -781,37 +834,12 @@ void KTextEdit::setReadOnly( bool readOnly )
 
 void KTextEdit::checkSpelling()
 {
-  if(document()->isEmpty())
-  {
-      KMessageBox::information(this, i18n("Nothing to spell check."));
-      return;
-  }
-  Sonnet::BackgroundChecker *backgroundSpellCheck = new Sonnet::BackgroundChecker;
-  if(!d->spellCheckingLanguage.isEmpty())
-     backgroundSpellCheck->changeLanguage(d->spellCheckingLanguage);
-  Sonnet::Dialog *spellDialog = new Sonnet::Dialog(
-      backgroundSpellCheck, 0);
-  backgroundSpellCheck->setParent(spellDialog);
-  spellDialog->setAttribute(Qt::WA_DeleteOnClose, true);
-  connect(spellDialog, SIGNAL(replace(QString,int,QString)),
-          this, SLOT(spellCheckerCorrected(QString,int,QString)));
-  connect(spellDialog, SIGNAL(misspelling(QString,int)),
-          this, SLOT(spellCheckerMisspelling(QString,int)));
-  connect(spellDialog, SIGNAL(autoCorrect(QString,QString)),
-          this, SLOT(spellCheckerAutoCorrect(QString,QString)));
-  connect(spellDialog, SIGNAL(done(QString)),
-          this, SLOT(spellCheckerFinished()));
-  connect(spellDialog, SIGNAL(cancel()),
-          this, SLOT(spellCheckerCanceled()));
-  connect(spellDialog, SIGNAL(stop()),
-          this, SLOT(spellCheckerFinished()));
-  connect(spellDialog, SIGNAL(spellCheckStatus(QString)),
-          this,SIGNAL(spellCheckStatus(QString)));
-  connect(spellDialog, SIGNAL(languageChanged(QString)),
-          this, SIGNAL(languageChanged(QString)));
-  d->originalDoc = QTextDocumentFragment(document());
-  spellDialog->setBuffer(toPlainText());
-  spellDialog->show();
+  d->checkSpelling(false);
+}
+
+void KTextEdit::forceSpellChecking()
+{
+  d->checkSpelling(true);
 }
 
 void KTextEdit::highlightWord( int length, int pos )
@@ -825,7 +853,7 @@ void KTextEdit::highlightWord( int length, int pos )
 
 void KTextEdit::replace()
 {
-     if( document()->isEmpty() )  // saves having to track the text changes
+     if ( document()->isEmpty() )  // saves having to track the text changes
         return;
 
     if ( d->repDlg ) {
@@ -976,7 +1004,7 @@ void KTextEdit::slotFindNext()
 
 void KTextEdit::slotFind()
 {
-    if( document()->isEmpty() )  // saves having to track the text changes
+    if ( document()->isEmpty() )  // saves having to track the text changes
         return;
 
     if ( d->findDlg ) {
@@ -991,7 +1019,7 @@ void KTextEdit::slotFind()
 
 void KTextEdit::slotReplace()
 {
-    if( document()->isEmpty() )  // saves having to track the text changes
+    if ( document()->isEmpty() )  // saves having to track the text changes
         return;
 
     if ( d->repDlg ) {
@@ -1009,6 +1037,11 @@ void KTextEdit::enableFindReplace( bool enabled )
     d->findReplaceEnabled = enabled;
 }
 
+void KTextEdit::showTabAction( bool show )
+{
+    d->showTabAction = show;
+}
+
 void KTextEdit::setSpellInterface(KTextEditSpellInterface *spellInterface)
 {
     d->spellInterface = spellInterface;
@@ -1128,4 +1161,19 @@ void KTextEdit::focusOutEvent(QFocusEvent *ev)
     QTextEdit::focusOutEvent(ev);
 }
 
+void KTextEdit::showAutoCorrectButton(bool show)
+{
+    d->showAutoCorrectionButton = show;
+}
+
+void KTextEdit::mousePopupMenuImplementation(const QPoint& pos)
+{
+    QMenu *popup = mousePopupMenu();
+    if ( popup ) {
+       aboutToShowContextMenu(popup);
+       popup->exec( pos );
+       delete popup;
+    }
+}
+
 #include "ktextedit.moc"
