@@ -107,7 +107,9 @@ private Q_SLOTS:
     void testOpenUrlTwiceWithKeep();
     void testOpenAndStop();
     void testBug211472();
+    void testRenameCurrentDir();
     void testRedirection();
+    void testWatchingAfterCopyJob();
     void testDeleteCurrentDir(); // must be last!
 
 protected Q_SLOTS: // 'more private than private slots' - i.e. not seen by qtestlib
